//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef PHRECT_HPP
#define PHRECT_HPP
//------------------------------------------------------------------------------
#include "PhysicObject.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A physic object representing a rectangle created using Group::CreatePhRect. */
/// LUA_HEADER_COPY_OFF ///
class PhRect : public PhysicObjectLine
{
public:
    PhRect(Group *parent, int phGroupId, const std::string &name, const Vector2D &size);
    static PhRect *Create(Group *parent, int phGroupId, const std::string &name, const Vector2D &size) {return new PhRect(parent, phGroupId, name, size);}

    /// LUA_EXPORT_ON(PhRect, PhysicObject) ///
    // Size
    /** Access the size of the rectangle. */
    //LUA// luaProperty(Vector2D, size, GetSize, SetSize);
    /** Set the size of the rectangle. */
    Vector2D GetSize() const;
    /** Get the size of the rectangle. */
    void SetSize(const Vector2D &size);

    /** Access the X size of the rectangle. */
    //LUA// luaProperty(float, sizeX, GetSizeX, SetSizeX);
    /** Access the Y size of the rectangle. */
    //LUA// luaProperty(float, sizeY, GetSizeY, SetSizeY);
    /** Get the X size of the rectangle. */
    float GetSizeX() const {return GetSize().x;}
    /** Get the Y size of the rectangle. */
    float GetSizeY() const {return GetSize().y;}
    /** Set the X size of the rectangle. */
    void SetSizeX(float size) {SetSize(Vector2D(size, GetSizeY()));} // RefreshMaxInterRadius(); in SetSize()
    /** Set the Y size of the rectangle. */
    void SetSizeY(float size) {SetSize(Vector2D(GetSizeX(), size));} // RefreshMaxInterRadius(); in SetSize()

    // Clone & Create
    /** Create a clone of the object and return an handler to the clone. */
    virtual PhRect *Clone() const {return CloneToGroup(NULL);}

    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual PhRect *CloneToGroup(Group *newParent) const {PhRect *obj = new PhRect(*this); obj->RegisterClone(newParent); return obj;}
    /// LUA_EXPORT_OFF ///

    // No Lua
    // Lua
    //static void RegisterClassLua(lua_State *myLuaState);

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_PHYSIC_RECT)
    virtual std::vector<Vector2D> GetAllPoints() const;

protected:
private:
    Vector2D m_size;
};
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
#endif // PHRECT_HPP
