//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef SCREENRENDERER_HPP
#define SCREENRENDERER_HPP
//------------------------------------------------------------------------------
#include "Drawable.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
class Screen;
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A screen renderer is used to draw the content of an other screen in the current screen. It can be usefull if you want to draw the content of your game in the menu background, for example.
  * This object is created using Group::CreateScreenRendere.
  */
/// LUA_HEADER_COPY_OFF ///
class ScreenRenderer : public Drawable
{
public:
    ScreenRenderer(Group *parent, const std::string &name);

    /// LUA_EXPORT_ON(ScreenRenderer, Drawable) ///

    // Clone & Create
    /** Create a clone of the object and return an handler to the clone. */
    virtual ScreenRenderer *Clone() const {return CloneToGroup(NULL);}

    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual ScreenRenderer *CloneToGroup(Group *newParent) const {ScreenRenderer *obj = new ScreenRenderer(*this); obj->RegisterClone(newParent); return obj;}

    // Screen
    /** Access drawed screen */
    //LUA// luaProperty(Screen, targetScreen, SetTargetScreen, GetTargetScreen);

    /** Set the target screen */
    void SetTargetScreen(Screen *targetScreen);

    /** Get the target screen */
    Screen *GetTargetScreen() const {return m_targetScreen;}

    /// LUA_EXPORT_OFF ///

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_DRAWABLE_SCREEN_RENDERER)

    // Render
    virtual void Render(sf::RenderTarget &App);

protected:
    Screen *m_targetScreen;
};
//----------------------------------------------------------------------------
} // namespace En

//------------------------------------------------------------------------------
#endif // SCREENRENDERER_HPP
