package.path = package.path .. ";" .. GetCommonDataPath() .. "/?.lua"

require("guiLib/guiLib")

-- Create screen
-- Const values
WIDTH = 1024
HEIGHT = 768

-- ScreenShot size
screenshotSize = Vector2D(117, 85)
screenshotBorder = 4

-- Buttons
btSize = Vector2D(125, 30)
btMargin = 15

local config = ScreenConfig(Vector2D(WIDTH, HEIGHT))
--config:SetFixedRatio(false)

menuScreen = CreateScreen("menuScreen", config)

------------------------------------------------------------
-- Resize
------------------------------------------------------------
savedResolution = Vector2D(-1, -1)

function menuOnFrame(frameTime, event)
	if savedResolution ~= GetRealWindowSize() then
		savedResolution = GetRealWindowSize()
	
		local newHeight = (savedResolution.y * WIDTH) / savedResolution.x
		HEIGHT = math.max(newHeight, 500)
		menuScreen:SetScreenConfig(ScreenConfig(Vector2D(WIDTH, newHeight)))
		
		menuScreen:FindByName("wallpaper"):SetSize(Vector2D(WIDTH, newHeight))
	end
end
menuScreen:RegisterFrameEvent(menuOnFrame)

------------------------------------------------------------
-- Background image
------------------------------------------------------------

local tmp = menuScreen:CreateSprite("wallpaper")
tmp:SetImage(backTex)
tmp:SetSize(Vector2D(WIDTH, HEIGHT))
tmp:SetZpos(-1000)

------------------------------------------------------------
-- Create the preview window
------------------------------------------------------------

previewWindow = menuScreen:CreateGroup("previewWindow")
windowInit(previewWindow, true)
windowSetTitle(previewWindow, _("Preview"))
windowHide(previewWindow)
previewWindow:FindByNameGroup("body"):CreateSprite("screenshot")

------------------------------------------------------------
-- Create the modes window
------------------------------------------------------------

function menuOnLunchMode(group)
	LoadMode(group:GetUserData("modeName"))
end

function menuOnSeeModeImage(group)
	local img = screenShotTbl[group:GetUserData("tblIndex")]

	local pos = Vector2D(100, 100)
	if windowIsVisible(previewWindow) then
		-- If already visible, we use the same place
		pos = previewWindow:GetPosition()
	end

	-- Size
	windowResize(previewWindow, pos, img:GetSize(), false, true)

	-- Content
	local bodyTmp = previewWindow:FindByNameGroup("body")
	local tmp = bodyTmp:FindByNameSprite("screenshot")
	tmp:SetImage(img)
	tmp:SetSize(img:GetSize())
	
	-- Show window
	windowMakeTop(previewWindow)
end

windowGrp = menuScreen:CreateGroup("modesWin")
windowInit(windowGrp, true)
windowSetTitle(windowGrp, _("Modes"))

-- Size
local inSize = windowResize(windowGrp, Vector2D(10.5, 78.8), Vector2D(800, 485), true)

-- Content
local body = windowGrp:FindByNameGroup("body")

local config = GetConfigManager()
local nextYPos = 0

screenShotTbl = {}

for i = 0, config:GetModeCount() - 1 do
	local info = config:GetModeInfo(i)

	if info.idName ~= "Menu" then
		local beginYPos = nextYPos

		-- Name	
		local text = body:CreateText("name_" .. tostring(i), info.fullName)
		text:SetPositionY(nextYPos)
		
		local txtSize = text:GetTextSize()
		
		nextYPos = nextYPos + txtSize.y + 10
		
		-- Screenshot
			-- Back
		local pos = Vector2D(0, nextYPos)
		local tmp = body:CreateShapeRectangle("screenshotBack_" .. tostring(i), pos, screenshotSize + Vector2D(screenshotBorder * 2, screenshotBorder * 2) + pos, Color(0, 0, 0), 0, Color(0, 0, 0))
		
			-- Img
		local isScreenshot = true
		local filename = info.path .. "/" .. info.screenshot
		if info.screenshot ~= "" then
			local file, msg
			file, msg = io.open(filename, "r")
			if not file then
				printErrorConsole("Can't open screenshot file : " .. msg)
				isScreenshot = false
			end
		else
			isScreenshot = false
		end
		
		if isScreenshot == false then
			filename = GetCommonDataPath() .. "/no_screenshot.png"
		end
		
		file = nil
		
		table.insert(screenShotTbl, Texture())
		screenShotTbl[#screenShotTbl]:LoadFromFile(filename)
		
		local tmp = body:CreateSprite("screenshot_" .. tostring(i))
		tmp:SetImage(screenShotTbl[#screenShotTbl])
		tmp:SetSize(screenshotSize)
		tmp:SetPosition(Vector2D(screenshotBorder, nextYPos + screenshotBorder))
		
		local screenshotBtsHeight = nextYPos + screenshotSize.y + screenshotBorder * 2 + 10
		
		-- Lunch bt
		local btSize = Vector2D(screenshotSize.x + screenshotBorder * 2, 30)
		local bt = body:CreateGroup("bt_" .. tostring(i))
		buttonControlCreate(bt, _("Launch"), btSize)
		buttonSetEventFunction(bt, menuOnLunchMode)
		bt:SetUserData("modeName", info.idName)
		buttonSetFont(bt, 16, Color(0, 0, 0))
		buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
		bt:SetPosition(Vector2D(0, screenshotBtsHeight))
		
		local screenshotBtsHeight = screenshotBtsHeight + btSize.y
		
		-- Image bt
		if isScreenshot then
			-- margin
			screenshotBtsHeight = screenshotBtsHeight + 10
		
			local bt = body:CreateGroup("btPrev_" .. tostring(i))
			buttonControlCreate(bt, _("Preview"), btSize)
			buttonSetEventFunction(bt, menuOnSeeModeImage)
			bt:SetUserData("tblIndex", #screenShotTbl)
			buttonSetFont(bt, 16, Color(0, 0, 0))
			buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
			bt:SetPosition(Vector2D(0, screenshotBtsHeight))
			
			screenshotBtsHeight = screenshotBtsHeight + btSize.y
		end
		
		-- Description
		local text = body:CreateText("description", info.description .. "\n\n" .. _("Auther: ") .. info.auther .. "\n" .. _("Version: ") .. info.version)
		text:SetSize(18)
		text:SetPosition(Vector2D(screenshotSize.x + screenshotBorder + 10, nextYPos))
		text:SetMaxWidth(inSize.x - (screenshotSize.x + screenshotBorder + 10))
		local descriptionHeight = nextYPos + text:GetTextSize().y
		
		-- Total height of screenshot and description
		nextYPos = math.max(descriptionHeight, screenshotBtsHeight) + 10
		
		if i ~= config:GetModeCount() - 1 then
			-- Separation bar
			local pos = Vector2D(0, nextYPos)
			local tmp = body:CreateShapeRectangle("screenshotBar_" .. tostring(i), pos, Vector2D(inSize.x, 5) + pos, Color(0, 0, 0), 0, Color(0, 0, 0))
			
			nextYPos = nextYPos + 10 + 5
		end
	end
end

-- Size
windowSetScrollHeight(windowGrp, nextYPos)
windowScroll(windowGrp, 0)
------------------------------------------------------------
-- Create the option window
------------------------------------------------------------

windowGrp = menuScreen:CreateGroup("optionsWin")
windowInit(windowGrp, true)
windowSetTitle(windowGrp, _("Options"))

-- Size
inSize = windowResize(windowGrp, Vector2D(300, 87.2), Vector2D(700, 300), false)

-- Content
windowSetScrollHeight(windowGrp, 0)
windowScroll(windowGrp, 0)

controlWidth = 150
controlHeight = 30
fontSize = 16
local body = windowGrp:FindByNameGroup("body")

	-- Fullscreen
local label = body:CreateText("label", _("Fullscreen"))
label:SetSize(fontSize)
label:SetPosition(Vector2D(0, 0 + controlHeight / 2 -label:GetTextSize().y / 2))
	
local control = body:CreateGroup("fullscreenCtrl")
choiceControlCreate(control, controlWidth, fontSize)
choiceControlAddStringTbl(control, {_("Yes"), _("No")})
control:SetPosition(Vector2D(inSize.x - controlWidth, 0))

	-- VSync
local label = body:CreateText("label", _("Enable vertical synchronisation"))
label:SetSize(fontSize)
label:SetPosition(Vector2D(0, 40 + controlHeight / 2 -label:GetTextSize().y / 2))
	
local control = body:CreateGroup("vSyncCtrl")
choiceControlCreate(control, controlWidth, fontSize)
choiceControlAddStringTbl(control, {_("Yes"), _("No")})
control:SetPosition(Vector2D(inSize.x - controlWidth, 40))

	-- Resolution
local label = body:CreateText("label", _("Resolution (auto: use the desktop resolution)"))
label:SetSize(fontSize)
label:SetPosition(Vector2D(0, 80 + controlHeight / 2 -label:GetTextSize().y / 2))
	
local control = body:CreateGroup("resolutionCtrl")
choiceControlCreate(control, controlWidth, fontSize)

resolutionTbl = GetAvaibleVideoModes()
for i = 1, #resolutionTbl do
	local str = ""
	if resolutionTbl[i] == Vector2D(-1, -1) then
		str = _("Auto")
	else
		str = tostring(resolutionTbl[i].x) .. "x" .. tostring(resolutionTbl[i].y)
	end
	
	choiceControlAddString(control, str)
end

control:SetPosition(Vector2D(inSize.x - controlWidth, 80))

	-- Anti Aliasing
local label = body:CreateText("label", _("Anti-aliasing level"))
label:SetSize(fontSize)
label:SetPosition(Vector2D(0, 120 + controlHeight / 2 -label:GetTextSize().y / 2))
	
local control = body:CreateGroup("antialiasingCtrl")
choiceControlCreate(control, controlWidth, fontSize)
for i = 0, 8 do	
	choiceControlAddString(control, tostring(i))
end
control:SetPosition(Vector2D(inSize.x - controlWidth, 120))

function loadConf(body)
	local node = GetConfigManager():GetGlobalConfigNode():AccessNode("graphicConf")
	
	-- Resolution
	local resolution = Vector2D(node:GetNumber("width"), node:GetNumber("height"))
	local resolutionCtrl = body:FindByNameGroup("resolutionCtrl")
	choiceControlChangeId(resolutionCtrl, 1) -- Auto
	for i = 2, #resolutionTbl do
		if resolutionTbl[i] == resolution then
			choiceControlChangeId(resolutionCtrl, i)
			i = #resolutionTbl + 1 -- Stop loop
		end
	end
	
	-- isVsync
	local isVsync = (node:GetNumber("isVsyn") == 1)
	if(isVsync) then
		choiceControlChangeId(body:FindByNameGroup("vSyncCtrl"), 1)
	else
		choiceControlChangeId(body:FindByNameGroup("vSyncCtrl"), 2)
	end
	
	-- isFullscreen
	local isFullscreen = (node:GetNumber("isFullscreen") == 1)
	if(isFullscreen) then
		choiceControlChangeId(body:FindByNameGroup("fullscreenCtrl"), 1)
	else
		choiceControlChangeId(body:FindByNameGroup("fullscreenCtrl"), 2)
	end
	
	-- antialiasingLevel
	local antialiasingLevel = node:GetNumber("antialiasingLevel")
	if(antialiasingLevel >= 0 and antialiasingLevel <= 8) then
		choiceControlChangeId(body:FindByNameGroup("antialiasingCtrl"), antialiasingLevel + 1)
	end
end
loadConf(body)

function saveConf(group)
	local node = GetConfigManager():GetGlobalConfigNode():AccessNode("graphicConf")
	
	-- Resolution
	local resolutionCtrl = body:FindByNameGroup("resolutionCtrl")
	resolution = resolutionTbl[choiceControlGetCurrentId(resolutionCtrl)]
	node:SetValue("width", resolution.x)
	node:SetValue("height", resolution.y)

	-- isVsync
	if choiceControlGetCurrentId(body:FindByNameGroup("vSyncCtrl")) == 1 then
		node:SetValue("isVsyn", 1)
	else
		node:SetValue("isVsyn", 0)
	end
	
	-- isFullscreen
	if choiceControlGetCurrentId(body:FindByNameGroup("fullscreenCtrl")) == 1 then
		node:SetValue("isFullscreen", 1)
	else
		node:SetValue("isFullscreen", 0)
	end

	-- antialiasingLevel
	node:SetValue("antialiasingLevel", choiceControlGetCurrentId(body:FindByNameGroup("antialiasingCtrl")) - 1)
end
saveConf(body)

function loadDefaultConf(group)
	local node = GetConfigManager():GetGlobalConfigNode():AccessNode("graphicConf")
	
	-- Resolution
	choiceControlChangeId(body:FindByNameGroup("resolutionCtrl"), 1) -- Auto
	
	-- isVsync
	choiceControlChangeId(body:FindByNameGroup("vSyncCtrl"), 1) -- true
	
	-- isFullscreen
	choiceControlChangeId(body:FindByNameGroup("fullscreenCtrl"), 1) -- true
	
	-- antialiasingLevel
	choiceControlChangeId(body:FindByNameGroup("antialiasingCtrl"), 1) -- 0
end

-- Bts events
function optionOnDefault(bt)
	local body = bt:GetParent()
	loadDefaultConf(body)
end

-- Bts events
function optionOnCancel(bt)
	local body = bt:GetParent()
	loadConf(body)
end

-- Bts events
function optionOnApply(bt)
	local body = bt:GetParent()
	saveConf(body)
	
	RequestRecreateRenderWindow()
end

-- Buttons
local bt = body:CreateGroup("btDefault")
buttonControlCreate(bt, _("Default"), btSize)
buttonSetEventFunction(bt, optionOnDefault)
buttonSetFont(bt, 16, Color(0, 0, 0))
buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
bt:SetPosition(inSize - Vector2D(btSize.x * 3 + btMargin * 2, btSize.y))

local bt = body:CreateGroup("btCancel")
buttonControlCreate(bt, _("Cancel"), btSize)
buttonSetEventFunction(bt, optionOnCancel)
buttonSetFont(bt, 16, Color(0, 0, 0))
buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
bt:SetPosition(inSize - Vector2D(btSize.x * 2 + btMargin, btSize.y))

local bt = body:CreateGroup("btApply")
buttonControlCreate(bt, _("Apply"), btSize)
buttonSetEventFunction(bt, optionOnApply)
buttonSetFont(bt, 16, Color(0, 0, 0))
buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
bt:SetPosition(inSize - Vector2D(btSize.x, btSize.y))


------------------------------------------------------------
-- Create the about window
------------------------------------------------------------

windowGrp = menuScreen:CreateGroup("aboutWin")
windowInit(windowGrp, true)
windowSetTitle(windowGrp, _("About"))

-- Size
inSize = windowResize(windowGrp, Vector2D(490, 150), Vector2D(450, 300), true)

-- Content
local bodyTmp = windowGrp:FindByNameGroup("body")
local img = bodyTmp:CreateSprite("user")
img:SetImage(designC501Tex)

local text = bodyTmp:CreateText("in", _("Programer: Colin GILLE\nDesigner: Alain Lhermitte"))
text:SetPositionY(img:GetSize().y + 50)
text:SetMaxWidth(inSize.x)
text:SetSize(16)
txtSize = text:GetTextSize()

windowSetScrollHeight(windowGrp, text:GetPosition().y + txtSize.y)

windowScroll(windowGrp, 0)

-- Create a test control
--[[local body = windowGrp:FindByNameGroup("body")

testControl1 = body:CreateGroup("testControl1")
choiceControlCreate(testControl1)
choiceControlAddStringTbl(testControl1, {"Coucou", "C'est", "moi"})
choiceControlAddString(testControl1, "++++")

testControl2 = body:CreateGroup("testControl2")
choiceControlCreate(testControl2)
choiceControlAddStringTbl(testControl2, {"800x600", "1024x768", "1280x1024", "1600x1200"})
testControl2:SetPosition(Vector2D(0, 70))]]

------------------------------------------------------------
-- Create the menu - Task bar
------------------------------------------------------------

function onQuitAnswer(button)
	local name = button:GetName()
	
	if name == "yesBt" then
		Quit()
	else
		local quitFgWindow = windowGetParentWindow(button)
		local quitBgWindow = quitFgWindow:GetParent():FindByNameGroup("quitBgWindow")
		
		windowDestroy(quitFgWindow)
		windowDestroy(quitBgWindow)
	end
end

function showQuitWindow()
	local quitFgWindow = menuScreen:FindByName("quitFgWindow")

	if quitFgWindow == nil then
		-- Background window
		quitBgWindow = menuScreen:CreateGroup("quitBgWindow")
		windowInit(quitBgWindow, false)
		windowSetBackgroundColors(quitBgWindow, Color(0, 0, 0, 20))
		windowMakeAlwaysActive(quitBgWindow, true)
		inSize = windowResize(quitBgWindow, Vector2D(0, 0), Vector2D(WIDTH, HEIGHT), false)
		
		local body = quitBgWindow:FindByNameGroup("body")
		local postFx = body:CreatePostFX("blurEffect")
		postFx:LoadFromFile(GetCommonDataPath() .. "/blur.sfx")

		
		-- Foreground window
		quitFgWindow = menuScreen:CreateGroup("quitFgWindow")
		windowInit(quitFgWindow, true)
		--windowSetBackgroundColors(quitFgWindow, Color(255, 0, 0, 170), Color(255, 0, 0, 170))
		windowSetTitle(quitFgWindow, _("Quit"))
		windowMakeAlwaysActive(quitFgWindow, true)
		windowSetCloseEventFunction(quitFgWindow, onQuitAnswer) -- Redirect close bt event
		local winSize = Vector2D(btSize.x * 2 + btMargin, btSize.y + 30)
		inSize = windowResize(quitFgWindow, Vector2D(WIDTH, HEIGHT) / 2 - winSize / 2, winSize, false, true)
		quitFgWindow:FindByNameGroup("body"):CreateSprite("screenshot")

		-- Content
		local body = quitFgWindow:FindByNameGroup("body")

		local label = body:CreateText("label", _("Do you really want to quit ?"))
		label:SetSize(fontSize)
		--label:SetPosition(Vector2D(0, 0 + controlHeight / 2 -label:GetTextSize().y / 2))

		-- bt
		local bt = body:CreateGroup("noBt")
		buttonControlCreate(bt, _("No"), btSize)
		buttonSetEventFunction(bt, onQuitAnswer)
		buttonSetFont(bt, 16, Color(0, 0, 0))
		buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
		bt:SetPosition(Vector2D(0, 30))

		local bt = body:CreateGroup("yesBt")
		buttonControlCreate(bt, _("Yes"), btSize)
		buttonSetEventFunction(bt, onQuitAnswer)
		buttonSetFont(bt, 16, Color(0, 0, 0))
		buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
		bt:SetPosition(Vector2D(btSize.x + btMargin, 30))
	else -- Cancel
		local quitBgWindow = quitFgWindow:GetParent():FindByNameGroup("quitBgWindow")
		
		windowDestroy(quitFgWindow)
		windowDestroy(quitBgWindow)
	end
end


-- Event quit
menuScreen:RegisterButtonEvent(function() menuScreen:PlaySound(sndBtClick); showQuitWindow(); end, "Escape", true)

function menuOnChangeMenu(button)
	if windowIsWinParentActive(button) then
		local name = button:GetName()
		name = string.sub(name, 0, string.len(name) - 2) -- The name of the zone is <name>Zone
		
		if name ~= 'quit' then
			windowMakeTop(menuScreen:FindByNameGroup(name .. 'Win'))
		else
			showQuitWindow()
		end
	end
end

menuGrp = menuScreen:CreateGroup("taskBar")
menuGrp:SetZpos(5000)
windowInit(menuGrp, false)
windowSetTitle(menuGrp, "Menu")
windowMakeAlwaysActive(menuGrp, true)
inSize = windowResize(menuGrp, Vector2D(0, 0), Vector2D(WIDTH, 30), false)

-- Content
local body = menuGrp:FindByNameGroup("body")

-- bt
local bt = body:CreateGroup("modesBt")
buttonControlCreate(bt, _("Modes"), Vector2D(150, inSize.y))
buttonSetEventFunction(bt, menuOnChangeMenu)
buttonSetFont(bt, 16, Color(0, 0, 0))
buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
bt:SetPosition(Vector2D(50, 0))

local bt = body:CreateGroup("optionsBt")
buttonControlCreate(bt, _("Options"), Vector2D(150, inSize.y))
buttonSetEventFunction(bt, menuOnChangeMenu)
buttonSetFont(bt, 16, Color(0, 0, 0))
buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
bt:SetPosition(Vector2D(250, 0))

local bt = body:CreateGroup("aboutBt")
buttonControlCreate(bt, _("About"), Vector2D(150, inSize.y))
buttonSetEventFunction(bt, menuOnChangeMenu)
buttonSetFont(bt, 16, Color(0, 0, 0))
buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
bt:SetPosition(Vector2D(450, 0))

local bt = body:CreateGroup("quitBt")
buttonControlCreate(bt, _("Quit"), Vector2D(150, inSize.y))
buttonSetEventFunction(bt, menuOnChangeMenu)
buttonSetFont(bt, 16, Color(0, 0, 0))
buttonSetImagePack(bt, btNormalTex, btOverTex, btClickTex)
bt:SetPosition(Vector2D(WIDTH - 150, 0))

dateText = body:CreateText("dateText", "A")
dateText:SetSize(20)
dateText:SetPosition(Vector2D(WIDTH - 300, inSize.y / 2 - dateText:GetTextSize().y / 2 - 2))

function menuOnClock(event)
	dateText:SetText(os.date(_("%H:%M:%S")))
end
menuOnClock() -- Refresh immediatly
menuScreen:RegisterTimer(menuOnClock, 1, true)

------------------------------------------------------------
-- Mode window => to window
------------------------------------------------------------
windowMakeTop(menuScreen:FindByName("optionsWin"))
windowMakeTop(menuScreen:FindByName("modesWin"))
